/**************************************************************************//**
*   @file   ad9838.c
*   @brief  Implementation of AD9838 Driver for NIOS processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 158
******************************************************************************/
#define AD9838_GLOBALS

/*****************************************************************************/
/* Include Files															 */
/*****************************************************************************/
#include "ad9838.h"
#include <altera_avalon_pio_regs.h>	//PIO access macros.
#include <altera_avalon_spi.h>		//SPI functions.
#include <unistd.h>					//usleep() function.
#include <system.h>
#if (USE_AD9838 == 1)

/*****************************************************************************/
/* Private variables                                                         */
/*****************************************************************************/
static unsigned int		controlRegister		= 0;						// Value of the Control Register.
static unsigned char	freqControlOld		= 0;						// Previous value of the freqControl variable.
static unsigned char	phaseControlOld		= 0;						// Previous value of the phaseControl variable.
static unsigned char	hwSwControlOld		= 0;						// Previous value of the hwSwControl variable.
static unsigned char	rampSinControlOld	= 0;						// Previous val of the rampSinControl variable.
static unsigned char	opbitenControlOld	= 0;						// Previous val of the opbitenControl variable.
static unsigned char	signpibControlOld	= 0;						// Previous val of the signpibControl variable.
static unsigned char	div2ControlOld		= 0;						// Previous value of the div2Control variable.
static unsigned char	sleepControlOld		= 3;						// Previous value of the sleepControl variable.
static unsigned char	sleep1				= 0;						// Value of the SLEEP1 bit.
static unsigned char	sleep12				= 0;						// Value of the SLEEP12 bit.
static unsigned char	sweepControlOld		= 0;						// Previous value of the sweepControl variable.
static unsigned char	slider0ValueOld		= 0;						// Previous value of the slider0Value variable.
static unsigned char	slider1ValueOld		= 0;						// Previous value of the slider1Value variable.
static unsigned int		slider2ValueOld		= 0;						// Previous value of the slider2Value variable.
static unsigned int		slider3ValueOld		= 0;						// Previous value of the slider3Value variable.
static unsigned char	slider4ValueOld		= 0;						// Previous value of the slider4Value variable.
static unsigned char	slider5ValueOld		= 0;						// Previous value of the slider5Value variable.
static unsigned char	slider6ValueOld		= 0;						// Previous value of the slider6Value variable.
unsigned char			txBuffer[4] 		= {0,};						// Tx SPI Buffer.
unsigned char   		rxBuffer[4] 		= {0,};						// Rx SPI Buffer.

/*****************************************************************************/
/*  Aliases for the uC Probe data                                            */
/*****************************************************************************/
volatile unsigned int*	frequency0Value		= &AD9838_data[0].uiData;	// Value of the frequency register 0.
volatile unsigned int*	frequency1Value		= &AD9838_data[1].uiData;	// Value of the frequency register 1.
volatile unsigned char*	freqControl			= &AD9838_data[2].uchData;	// Select the frequency register.
volatile unsigned int*	phase0Value			= &AD9838_data[3].uiData;	// Value of the phase register 0.
volatile unsigned int*	phase1Value			= &AD9838_data[4].uiData;	// Value of the phase register 1.
volatile unsigned char*	phaseControl		= &AD9838_data[5].uchData;	// Select the phase register.
volatile unsigned char*	hwSwControl			= &AD9838_data[6].uchData;	// Bit / Pin selection.
volatile unsigned char*	rampSinControl		= &AD9838_data[7].uchData;	// Sinusoidal output / Triangle output.
volatile unsigned char*	sleepControl		= &AD9838_data[8].uchData;	// Select the sleep mode.
volatile unsigned char*	opbitenControl		= &AD9838_data[9].uchData;	// Enables / disables the logic output.
volatile unsigned char*	signpibControl		= &AD9838_data[10].uchData;	// Connects comparator/MSB to SIGN BIT OUT pin.
volatile unsigned char*	div2Control			= &AD9838_data[11].uchData;	// MSB / MSB/2.
volatile unsigned char*	sweepControl		= &AD9838_data[12].uchData;	// Enables the sweep mode.
volatile unsigned char* startSweepControl	= &AD9838_data[13].uchData;	// Start sweep mode.
volatile unsigned int*	startFreqValue		= &AD9838_data[14].uiData;	// Value of the start frequency.
volatile unsigned int*	stopFreqValue		= &AD9838_data[15].uiData;	// Value of the stop frequency.
volatile unsigned int*	stepFreqValue		= &AD9838_data[16].uiData;	// Value of the step frequency.
volatile unsigned int*	sweepDelayValue		= &AD9838_data[17].uiData;	// Value of the sweep delay.
volatile unsigned char* freqPin				= &AD9838_data[18].uchData;	// Value of the Frequency Pin.
volatile unsigned char* phasePin			= &AD9838_data[19].uchData;	// Value of the Phase Pin.
volatile unsigned char* sleepPin			= &AD9838_data[20].uchData;	// Value of the Sleep Pin.
volatile unsigned char* resetPin			= &AD9838_data[21].uchData;	// Value of the Reset Pin.
volatile unsigned char* dial0Value			= &AD9838_data[22].uchData;	// Value of the Frequency 0 Dial.
volatile unsigned char* slider0Value		= &AD9838_data[23].uchData;	// Value of the Frequency 0 Slider.
volatile unsigned char* dial1Value			= &AD9838_data[24].uchData;	// Value of the Frequency 1 Dial.
volatile unsigned char* slider1Value		= &AD9838_data[25].uchData;	// Value of the Frequency 1 Slider.
volatile unsigned int*	slider2Value		= &AD9838_data[26].uiData;	// Value of the Phase 0 Slider.
volatile unsigned int*	slider3Value		= &AD9838_data[27].uiData;	// Value of the Phase 1 Slider.
volatile unsigned char* dial4Value			= &AD9838_data[28].uchData;	// Value of the Start Frequency Dial.
volatile unsigned char* slider4Value		= &AD9838_data[29].uchData;	// Value of the Start Frequency Slider.
volatile unsigned char* dial5Value			= &AD9838_data[30].uchData;	// Value of the Stop Frequency Dial.
volatile unsigned char* slider5Value		= &AD9838_data[31].uchData;	// Value of the Stop Frequency Slider.
volatile unsigned char* dial6Value			= &AD9838_data[32].uchData;	// Value of the Step Frequency Dial.
volatile unsigned char* slider6Value		= &AD9838_data[33].uchData;	// Value of the Step Frequency Slider.

/**************************************************************************//**
 * @brief Creates a delay.
 *
 * @param ms - Number of milliseconds.
 *
 * @return None.
******************************************************************************/
void AD9838_DelayMs(unsigned int ms)
{
	unsigned int counter;
	for(counter = 0; counter < ms; counter ++)
	{
		usleep(1000);
	}
}

/**************************************************************************//**
 * @brief Transmits 16 bits on SPI.
 *
 * @param value - Data which will be transmitted.
 *
 * @return None
******************************************************************************/
void AD9838_TxSpi(unsigned int value)
{
	txBuffer[0] = (unsigned char) ((value & 0x0000ff00) >> 8);
	txBuffer[1] = (unsigned char)  (value & 0x000000ff);
    alt_avalon_spi_command(AD9838_SPI,  // base address of SPI module
    					   0,           // slave number
    					   2,           // write_length
    					   txBuffer,    // *write_data
    					   0,           // read_length
    					   rxBuffer,    // *read_data
    					   0);          // flags
}

/**************************************************************************//**
 * @brief Initialize SPI and Initial Values for AD9838 Board.
 *
 * @param None.
 *
 * @return SUCCESS.
******************************************************************************/
int AD9838_Init(void)
{
	unsigned int spiData = 0;
	*frequency0Value	= 0;
	*frequency1Value	= 0;
	*freqControl		= 0;
	*phase0Value		= 0;
	*phase1Value		= 0;
	*phaseControl		= 0;
	*hwSwControl		= 0;
	*rampSinControl		= 0;
	*sleepControl		= 3;
	*opbitenControl		= 0;
	*signpibControl		= 0;
	*div2Control		= 0;
	*sweepControl		= 0;
	*startSweepControl	= 0;
	*startFreqValue		= 0;
	*stopFreqValue		= 0;
	*stepFreqValue		= 0;
	*sweepDelayValue	= 0;
	*freqPin			= 0;
	*phasePin			= 0;
	*sleepPin			= 0;
	*resetPin			= 0;
	*dial0Value			= 0;
	*slider0Value		= 0;
	*dial1Value			= 0;
	*slider1Value		= 0;
	*slider2Value		= 0;
	*slider3Value		= 0;
	*dial4Value			= 0;
	*slider4Value		= 0;
	*dial5Value			= 0;
	*slider5Value		= 0;
	*dial6Value			= 0;
	*slider6Value		= 0;
	IOWR_ALTERA_AVALON_PIO_DATA(GPIO_BASE, 0);				// Reset data.
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, GPIO_PSEL |
												GPIO_FSEL |
												GPIO_RESET |
												GPIO_SLEEP);// Output.
	SetCtrl(AD9838_SPI_CFG);								// Setup the SPI interface.
	spiData |= BIT_RESET;									// When the AD9838 is powered up, the part should be reset.
	AD9838_TxSpi(spiData);
	AD9838_DelayMs(10);
	spiData &=~ BIT_RESET;
	AD9838_TxSpi(spiData);
	AD9838_SetFreq(0,*frequency0Value);
	AD9838_SetFreq(1,*frequency1Value);
	AD9838_SetPhase(0,*phase0Value);
	AD9838_SetPhase(1,*phase1Value);

	return(SUCCESS);
}

/**************************************************************************//**
 * @brief Loads a frequency value on a register.
 *
 * @param registerNumber: Number of the register (0 / 1).
 * @param frequencyValue: Frequency value.
 *
 * @return None.
******************************************************************************/
void AD9838_SetFreq(unsigned char registerNumber, unsigned int frequencyValue)
{
	unsigned long int freqRegister;
	unsigned int freqLsb, freqMsb;
	unsigned int spiData = 0;
	freqRegister = frequencyValue * 16.777216f;
	freqLsb = (freqRegister & 0x0003FFF);
	freqMsb = ((freqRegister & 0xFFFC000) >> 14);
	spiData |= BIT_B28;
	AD9838_TxSpi(spiData);
	if(registerNumber == 0)
	{
		AD9838_TxSpi(BIT_F0ADDRESS + freqLsb);
		AD9838_TxSpi(BIT_F0ADDRESS + freqMsb);
	}
	else
	{
		AD9838_TxSpi(BIT_F1ADDRESS + freqLsb);
		AD9838_TxSpi(BIT_F1ADDRESS + freqMsb);
	}
}

/**************************************************************************//**
 * @brief Loads a phase value on a register.
 *
 * @param registerNumber: Number of the register (0 / 1).
 * @param phaseValue: Phase value.
 *
 * @return none
******************************************************************************/
void AD9838_SetPhase(unsigned char registerNumber, unsigned int phaseValue)
{
	if(registerNumber == 0)
	{
		AD9838_TxSpi(BIT_P0ADDRESS + phaseValue);
	}
	else
	{
		AD9838_TxSpi(BIT_P1ADDRESS + phaseValue);
	}
}

/**************************************************************************//**
 * @brief Configures the control register for current output.
 *
 * @param pinSw - Bit / Pin selection.
 * @param fSel - Selects the register number (0 / 1) for the frequency.
 * @param pSel - Selects the phase number (0 / 1) for the phase.
 * @param mode - Sinusoidal output / Triangle output.
 *
 * @return spiData - Value which can be load in the control register.
******************************************************************************/
unsigned int AD9838_CurrentOutput(unsigned char pinSw,
								  unsigned char fSel,
								  unsigned char pSel,
								  unsigned char mode)
{
	unsigned int spiData = 0;
	if (pinSw)						// Control pins implement the register selection,
	{								// reset, and DAC power-down functions.
		spiData |= BIT_PINSW;		// BIT_PINSW = 1.
		if(mode)
		{
			spiData |= BIT_MODE;	// BIT_MODE = 1.
		}
		else
		{
			spiData &=~ BIT_MODE;	// BIT_MODE = 0.
		}
	}
	else							// Control bits implement the register selection,
	{								// reset, and DAC power-down functions.
		spiData &=~ BIT_PINSW;		// BIT_PINSW = 0.
		if (fSel)
		{
			spiData |= BIT_FSEL;	// BIT_FSEL = 1.
		}
		else
		{
			spiData &=~ BIT_FSEL;	// BIT_FSEL = 0.
		}
		if (pSel)
		{
			spiData |= BIT_PSEL;	// BIT_PSEL = 1.
		}
		else
		{
			spiData &=~ BIT_PSEL;	// BIT_PSEL = 0.
		}
		if(mode)
		{
			spiData |= BIT_MODE;	// BIT_MODE = 1.
		}
		else
		{
			spiData &=~ BIT_MODE;	// BIT_MODE = 0.
		}
	}

	return(spiData);
}

/**************************************************************************//**
 * @brief Configures the control register for logic output.
 *
 * @param opbiten - Enables / disables the logic output.
 * @param signpib - Connects comparator / MSB to the SIGN BIT OUT pin.
 * @param div2 - MSB / MSB/2
 *
 * @return spiData - Value which can be load in the control register.
******************************************************************************/
unsigned int AD9838_LogicOutput(unsigned char opbiten,
								unsigned char signpib,
								unsigned char div2)
{
	unsigned int spiData = 0;
	if(opbiten)
	{
		spiData |= BIT_OPBITEN;		// BIT_OPBITEN = 1.
		if (signpib)
		{
			spiData |= BIT_SIGNPIB;	// BIT_SIGNPIB = 1.
		}
		else
		{
			spiData &=~ BIT_SIGNPIB;// BIT_SIGNPIB = 0.
		}
		if (div2)
		{
			spiData |= BIT_DIV2;	// BIT_DIV2 = 1.
		}
		else
		{
			spiData &=~ BIT_DIV2;	// BIT_DIV2 = 0.
		}
	}
	else
	{
		spiData &=~ BIT_OPBITEN;	// BIT_OPBITEN = 0.
	}

	return(spiData);
}

/**************************************************************************//**
 * @brief: Configures the control register for sleep mode.
 *
 * @param sleep1 - Enables / Disables the internal MCLK.
 * @param sleep12 - Powers up / Powers down the DAC.
 *
 * @return spiData - Value which can be load in the control register.
******************************************************************************/
unsigned int AD9838_SleepMode(unsigned char sleep1, unsigned char sleep12)
{
	unsigned int spiData = 0;
	if (sleep1)
	{
		spiData |= BIT_SLEEP1;		// BIT_SLEEP1 = 1.
	}
	else
	{
		spiData &=~ BIT_SLEEP1;		// BIT_SLEEP1 = 0.
	}
	if(sleep12)
	{
		spiData |= BIT_SLEEP12;		// BIT_SLEEP12 = 1.
	}
	else
	{
		spiData &=~ BIT_SLEEP12;	// BIT_SLEEP12 = 0.
	}

	return(spiData);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return Always returns success
******************************************************************************/
int AD9838_UpdateParameters(void)
{
	unsigned int sweepFrequency;
	unsigned int multiply;
	unsigned int tempCounter;
	if(*sweepControl)						// Sweep Mode.
	{
		if(sweepControlOld == 0)
		{
			AD9838_SetFreq(0,0);
			controlRegister = AD9838_CurrentOutput(0,0,0,0);
			sweepControlOld = 1;
		}
		AD9838_TxSpi(controlRegister);
		if(*startSweepControl)				// If the START switch is pressed, generates sweep frequency.
		{
			sweepFrequency = *startFreqValue;
			while((sweepFrequency < *stopFreqValue) &&
				  (*startSweepControl != 0))
			{
				sweepFrequency += *stepFreqValue;
				AD9838_SetFreq(0, sweepFrequency);
				controlRegister = AD9838_CurrentOutput(0,0,0,0);
				AD9838_TxSpi(controlRegister);
				AD9838_DelayMs(*sweepDelayValue);
			}
			AD9838_SetFreq(0,0);
			controlRegister = AD9838_CurrentOutput(0,0,0,0);
			*startSweepControl = 0;
		}
		if(slider4ValueOld != *slider4Value)// If the value of slider 4 has changed,
		{									// calculates the new startFreqValue.
			multiply = 1;
			for(tempCounter = 0; tempCounter < *dial4Value; tempCounter ++)
			{
				multiply = multiply * 10;
			}
			*startFreqValue = *startFreqValue - ((*startFreqValue % (multiply * 10)) / multiply) * multiply;
			*startFreqValue = *startFreqValue + (*slider4Value * multiply);
			slider4ValueOld = *slider4Value;
			if(*startFreqValue > 16000000)
			{
				*startFreqValue = 16000000;
			}
		}
		if(slider5ValueOld != *slider5Value)// If the value of slider 5 has changed,
		{									// calculates a new stopFreqValue.
			multiply = 1;
			for(tempCounter = 0; tempCounter < *dial5Value; tempCounter ++)
			{
				multiply = multiply * 10;
			}
			*stopFreqValue = *stopFreqValue -
					((*stopFreqValue % (multiply * 10)) / multiply) * multiply;
			*stopFreqValue = *stopFreqValue + (*slider5Value * multiply);
			slider5ValueOld = *slider5Value;
			if(*stopFreqValue > 16000000)
			{
				*stopFreqValue = 16000000;
			}
		}
		if(slider6ValueOld != *slider6Value)// If the value of slider 6 has changed,
		{									// calculates a new stepFreqValue.
			multiply = 1;
			for(tempCounter = 0; tempCounter < *dial6Value; tempCounter ++)
			{
				multiply = multiply * 10;
			}
			*stepFreqValue = *stepFreqValue -
					((*stepFreqValue % (multiply * 10)) / multiply) * multiply;
			*stepFreqValue = *stepFreqValue + (*slider6Value * multiply);
			slider6ValueOld = *slider6Value;
			if(*stepFreqValue > 16000000)
			{
				*stepFreqValue = 16000000;
			}
		}
	}
	else	// Normal Mode.
	{
		if(*freqPin)						// If the Frequency Pin Button is pressed,
		{									// set the frequency pin.
			IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_FSEL);
		}
		else
		{
			IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_FSEL);
		}
		if(*phasePin)						// If the Phase Pin Button is pressed,
		{									// set the phase pin.
			IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_PSEL);
		}
		else
		{
			IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_PSEL);
		}
		if(*sleepPin)						// If the Sleep Pin Button is pressed set the sleep pin.
		{
			IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_SLEEP);
		}
		else
		{
			IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_SLEEP);
		}
		if(*resetPin)						// If the Reset Pin Button is pressed set the reset pin.
		{
			IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_RESET);
		}
		else
		{
			IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_RESET);
		}
		if(sweepControlOld)					// After exiting the Sweep Mode,
		{									// set the frequency from Frequency Slider.
			AD9838_SetFreq(0,*frequency0Value);
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
			sweepControlOld = 0;
		}
		if(slider0ValueOld != *slider0Value)// If the value of slider 0 has changed,
		{									// calculates a new frequency0Value.
			multiply = 1;
			for(tempCounter = 0; tempCounter < *dial0Value; tempCounter ++)
			{
				multiply = multiply * 10;
			}
			*frequency0Value = *frequency0Value -
				((*frequency0Value % (multiply * 10)) / multiply) * multiply;
			*frequency0Value = *frequency0Value + (*slider0Value * multiply);
			slider0ValueOld = *slider0Value;
			if(*frequency0Value > 16000000)
			{
				*frequency0Value = 16000000;
			}
			AD9838_SetFreq(0,*frequency0Value);
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
							  AD9838_LogicOutput(*opbitenControl,
									  	  	  	 *signpibControl,
									  	  	  	 *div2Control) +
							  AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(slider1ValueOld != *slider1Value)// If the value of slider 0 has changed,
		{									// calculates a new frequency1Value.
			multiply = 1;
			for(tempCounter = 0; tempCounter < *dial1Value; tempCounter ++)
			{
				multiply = multiply * 10;
			}
			*frequency1Value = *frequency1Value -
				((*frequency1Value % (multiply * 10)) / multiply) * multiply;
			*frequency1Value = *frequency1Value + (*slider1Value * multiply);
			slider1ValueOld = *slider1Value;
			if(*frequency1Value > 16000000)
			{
				*frequency1Value = 16000000;
			}
			AD9838_SetFreq(1,*frequency1Value);
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												  *freqControl,
												  *phaseControl,
												  *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(freqControlOld != *freqControl)	// If the value of frequency control switch has changed,
		{								  	// selects the other frequency register.
			freqControlOld = *freqControl;
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(slider2ValueOld != *slider2Value)// If the value of slider 2 has changed,
		{									// calculates the new phase0Value.
			slider2ValueOld = *slider2Value;
			*phase0Value = *slider2Value * 11.37777778f;
			AD9838_SetPhase(0,*phase0Value);
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(slider3ValueOld != *slider3Value)// If the value of slider 3 has changed,
		{									// calculates the new phase1Value.
			slider3ValueOld = *slider3Value;
			*phase1Value = *slider3Value * 11.37777778f;
			AD9838_SetPhase(1,*phase1Value);
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(phaseControlOld != *phaseControl)// If the value of phase control switch has changed,
		{									// selects the other phase register.
			phaseControlOld = *phaseControl;
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(hwSwControlOld != *hwSwControl)	// If the value of PIN/SW switch has changed,
		{									// change the bit PIN/SW in the Control Register.
			hwSwControlOld = *hwSwControl;
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(rampSinControlOld != *rampSinControl)// If the value of Mode switch has changed,
		{										// change the bit MODE in the Control Register.
			rampSinControlOld = *rampSinControl;
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(opbitenControlOld != *opbitenControl)// If the value of OPBITEN switch has changed,
		{										// change the bit OPBITEN in the Control Register.
			opbitenControlOld = *opbitenControl;
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(signpibControlOld != *signpibControl)// If the value of SIGN/PIB switch has changed,
		{										// change the bit SIGN/PIB in the Control Register.
			signpibControlOld = *signpibControl;
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(div2ControlOld != *div2Control)		// If the value of the DIV2 switch has changed,
		{										// change the bit DIV2 in the Control Register.
			div2ControlOld = *div2Control;
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control) +
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
		if(sleepControlOld != *sleepControl)	// If the value of the Sleep slider has changed,
		{										// change the bits SLEEP1 and SLEEP12 in the Control Register.
			sleepControlOld = *sleepControl;
			if(*sleepControl == 3)
			{
				sleep1 = 0;
				sleep12 = 0;
			}
			if(*sleepControl == 2)
			{
				sleep1 = 1;
				sleep12 = 0;
			}
			if(*sleepControl == 1)
			{
				sleep1 = 0;
				sleep12 = 1;
			}
			if(*sleepControl == 0)
			{
				sleep1 = 1;
				sleep12 = 1;
			}
			controlRegister = AD9838_CurrentOutput(*hwSwControl,
												   *freqControl,
												   *phaseControl,
												   *rampSinControl) +
								AD9838_LogicOutput(*opbitenControl,
												   *signpibControl,
												   *div2Control)+
								AD9838_SleepMode(sleep1, sleep12);
			AD9838_TxSpi(controlRegister);
		}
	}

	return(SUCCESS);
}
#endif /* USE_AD9838 */
