/**************************************************************************//**
*   @file   ad9838.h
*   @brief  Header file of AD9838 Driver for NIOS processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 158
******************************************************************************/
#ifndef _AD9838_H_
#define _AD9838_H_
#ifdef AD9838_GLOBALS
	#define AD9838_EXT extern volatile
#else
	#define AD9838_EXT volatile
#endif

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "adi.h"
#if (USE_AD9838 == 1)

/*****************************************************************************/
/* SPI Configuration                                                         */
/*****************************************************************************/
#define AD9838_SPI		SPI_1_P0_BASE
#define AD9838_SPI_CFG	(SPI1_ena | SPI1_inv)

/*****************************************************************************/
/* GPIO                                                                      */
/*****************************************************************************/
#define GPIO_PSEL		(1<<0)		// PSEL Pin.
#define GPIO_FSEL		(1<<1)		// FSEL Pin.
#define GPIO_RESET		(1<<2)		// RESET Pin.
#define GPIO_SLEEP		(1<<3)		// SLEEP Pin.

/*****************************************************************************/
/* Control Register Bits                                                     */
/*****************************************************************************/
#define BIT_MODE		(1 << 1)	// D1.
#define BIT_DIV2		(1 << 3)	// D3.
#define BIT_SIGNPIB		(1 << 4)	// D4.
#define BIT_OPBITEN		(1 << 5)	// D5.
#define BIT_SLEEP1		(1 << 6)	// D6.
#define BIT_SLEEP12		(1 << 7)	// D7.
#define BIT_RESET		(1 << 8)	// D8.
#define BIT_PINSW		(1 << 9)	// D9.
#define BIT_PSEL		(1 << 10)	// D10.
#define BIT_FSEL		(1 << 11)	// D11.
#define BIT_B28			(1 << 13)	// D13.
#define BIT_F0ADDRESS	0x4000		// Frequency Register 0 address.
#define BIT_F1ADDRESS	0x8000		// Frequency Register 1 address.
#define BIT_P0ADDRESS	0xC000		// Phase Register 0 address.
#define BIT_P1ADDRESS	0xE000		// Phase Register 1 address.

/*****************************************************************************/
/* Monitored variables declarations                                          */
/*****************************************************************************/
AD9838_EXT uCProbeDataUnion AD9838_data[34];	// Used to store the values for uC-Probe communication.
#endif //(USE_AD9838 == 1)

/*****************************************************************************/
/* Functions Prototypes                                                      */
/*****************************************************************************/
int AD9838_Init(void); 									// Initializes the communication with the device.
int AD9838_UpdateParameters(void); 						// Updates the device's monitored and controlled parameters.
void AD9838_TxSpi(unsigned int value); 					// Transmits 16 bits on SPI.
void AD9838_SetFreq(unsigned char registerNumber,
					unsigned int frequencyValue); 		// Loads a frequency value on a register.
void AD9838_SetPhase(unsigned char registerNumber,
					 unsigned int phaseValue); 			// Loads a phase value on a register.
unsigned int AD9838_CurrentOutput(unsigned char pinSw,
								  unsigned char fSel,
								  unsigned char pSel,
								  unsigned char mode); 	// Configures the control register for current output.
unsigned int AD9838_LogicOutput(unsigned char opbiten,
								unsigned char signpib,
								unsigned char div2); 	// Configures the control register for logic output.
unsigned int AD9838_SleepMode(unsigned char sleep1,
							  unsigned char sleep12); 	// Configures the control register for sleep mode.
void AD9838_DelayMs(unsigned int ms); 					// Creates a delay.
#endif /* AD9838_H_ */
